;!function(){try{var e="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:{},n=(new Error).stack;n&&(e._btDebugIds=e._btDebugIds||{},e._btDebugIds[n]="de9639da-78ea-6158-629f-3617b30ed539")}catch(e){}}();
import PackageData from '../../../package.json' assert { type: 'json' }; // eslint-disable-line
import { RESOURCES_ERR } from "../../app/error/error-codes.js";
import { iCPSError } from "../../app/error/error.js";
import { EventManager } from "./event-manager.js";
import { iCPSEventLog } from "./events-types.js";
import { NetworkManager } from "./network-manager.js";
import { ResourceManager } from "./resource-manager.js";
import { Validator } from "./validator.js";
/**
 * This namespace handles the static access to the singleton functions of the ResourceManager, NetworkManager, Validator and EventManager
 */
export var Resources;
(function (Resources) {
    /**
     * Static package info
     */
    Resources.PackageInfo = {
        name: PackageData.name,
        version: PackageData.version,
        description: PackageData.description,
    };
    /**
     * Prepares the global singleton instances. This includes the ResourceManager, NetworkManager, Validator and EventManager.
     * This function should only be called once.
     * @param appOptions - The parsed app options
     * @returns The singleton instances
     * @throws If the function is called with an already initiated singleton
     */
    function setup(appOptions) {
        if (Resources._instances) {
            throw new iCPSError(RESOURCES_ERR.ALREADY_INITIATED);
        }
        Resources._instances = {}; // Creating one-by one, so they are available as soon as possible
        Resources._instances.event = new EventManager();
        Resources._instances.validator = new Validator();
        Resources._instances.manager = new ResourceManager(appOptions);
        Resources._instances.network = new NetworkManager(appOptions);
        return Resources._instances;
    }
    Resources.setup = setup;
    /**
     * @returns The singleton instances of the shared resources
     * @throws If the function is called before the singleton is initiated
     */
    function instances() {
        if (!Resources._instances) {
            throw new iCPSError(RESOURCES_ERR.NOT_INITIATED);
        }
        return Resources._instances;
    }
    Resources.instances = instances;
    /**
     * @returns The singleton instances of the shared event manager
     * @throws If the function is called before the singleton is initiated
     */
    function event() {
        const { event } = instances();
        if (!event) {
            throw new iCPSError(RESOURCES_ERR.EVENT_NOT_INITIATED);
        }
        return event;
    }
    Resources.event = event;
    /**
     * @returns The singleton instances of the shared network manager
     * @throws If the function is called before the singleton is initiated
     */
    function network() {
        const { network } = instances();
        if (!network) {
            throw new iCPSError(RESOURCES_ERR.NETWORK_NOT_INITIATED);
        }
        return network;
    }
    Resources.network = network;
    /**
     * @returns The singleton instances of the shared resource manager
     * @throws If the function is called before the singleton is initiated
     */
    function manager() {
        const { manager } = instances();
        if (!manager) {
            throw new iCPSError(RESOURCES_ERR.RESOURCE_NOT_INITIATED);
        }
        return manager;
    }
    Resources.manager = manager;
    /**
     * @returns The singleton instances of the shared validator
     * @throws If the function is called before the singleton is initiated
     */
    function validator() {
        const { validator } = instances();
        if (!validator) {
            throw new iCPSError(RESOURCES_ERR.VALIDATOR_NOT_INITIATED);
        }
        return validator;
    }
    Resources.validator = validator;
    /**
     * Returns a logger interface, that allows emitting log events to the event bus by providing a source object
     * Used to log messages associated to the source object
     * @param source - The source of the log message - either an object or a string
     * @returns The logger interface
     */
    function logger(source) {
        const format = function (...args) {
            return args.map(arg => String(arg)).join(` `);
        };
        return {
            log(...args) {
                Resources.event().emit(iCPSEventLog.INFO, source, format(args));
            },
            debug(...args) {
                Resources.event().emit(iCPSEventLog.DEBUG, source, format(args));
            },
            info(...args) {
                Resources.event().emit(iCPSEventLog.INFO, source, format(args));
            },
            warn(...args) {
                Resources.event().emit(iCPSEventLog.WARN, source, format(args));
            },
            error(...args) {
                Resources.event().emit(iCPSEventLog.ERROR, source, format(args));
            },
        };
    }
    Resources.logger = logger;
    /**
     * Returns an event interface, that allows listening to events emitted on the event bus, while binding the listener to the registry
     * Used to manage listeners for the listener object
     * @param listenerObject - The source of the registration request, used to track the listeners and enable cleanup
     * @returns The events interface
     */
    function events(listenerObject) {
        return {
            on(event, listener) {
                Resources.event().on(listenerObject, event, listener);
                return Resources.events(listenerObject); // Returning for chaining
            },
            once(event, listener) {
                Resources.event().once(listenerObject, event, listener);
                return Resources.events(listenerObject); // Returning for chaining
            },
            removeListeners(event) {
                Resources.event().removeListenersFromRegistry(listenerObject, event);
                return Resources.events(listenerObject); // Returning for chaining
            },
        };
    }
    Resources.events = events;
    /**
     * Emits an event on the central event bus
     * @param event - The event to emit
     * @param args - Optional arguments to pass to the event
     * @returns True if the event had listeners
     */
    function emit(event, ...args) {
        return Resources.event().emit(event, ...args);
    }
    Resources.emit = emit;
    /**
     * Typing namespace
     */
    let Types;
    (function (Types) {
        /**
         * Possible regions for this tool to operate in
         */
        let Region;
        (function (Region) {
            /**
             * Will use icloud.com
             */
            Region["WORLD"] = "world";
            /**
             * Will use icloud.com.cn
             */
            Region["CHINA"] = "china";
        })(Region = Types.Region || (Types.Region = {}));
    })(Types = Resources.Types || (Resources.Types = {}));
})(Resources || (Resources = {}));
//# sourceMappingURL=main.js.map
//# debugId=de9639da-78ea-6158-629f-3617b30ed539