;!function(){try{var e="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:{},n=(new Error).stack;n&&(e._btDebugIds=e._btDebugIds||{},e._btDebugIds[n]="936c64ec-57e0-e75f-223e-5c48e1c759fe")}catch(e){}}();
import { default as Ajv } from 'ajv';
import ResourceFileSchema from "./schemas/resource-file.json" assert { type: "json" }; // eslint-disable-line
import SigninResponseSchema from "./schemas/signin-response.json" assert { type: "json" }; // eslint-disable-line
import TrustResponseSchema from "./schemas/trust-response.json" assert { type: "json" }; // eslint-disable-line
import SetupResponseSchema from "./schemas/setup-response.json" assert { type: "json" }; // eslint-disable-line
import PhotosSetupResponseSchema from "./schemas/photos-setup-response.json" assert { type: "json" }; // eslint-disable-line
import ResendMFADeviceResponseSchema from "./schemas/resend-mfa-device-response.json" assert { type: "json" }; // eslint-disable-line
import ResendMFAPhoneResponseSchema from "./schemas/resend-mfa-phone-response.json" assert { type: "json" }; // eslint-disable-line
import PCSResponseSchema from "./schemas/pcs-response.json" assert { type: "json" }; // eslint-disable-line
import { iCPSError } from "../../app/error/error.js";
import { VALIDATOR_ERR } from "../../app/error/error-codes.js";
import { COOKIE_KEYS } from "./network-types.js";
/**
 * Common configuration for the schema validator
 */
const AJV_CONF = {
    verbose: true,
    // Logger: ResourceManager.logger(`AjvValidator`),
};
/**
 * This class is responsible for validating 3rd party provided JSON based resources using previously compiled JSON schemas
 */
export class Validator {
    /**
     * Validator for the resource file schema
     */
    _resourceFileValidator = new Ajv.default(AJV_CONF).compile(ResourceFileSchema);
    /**
     * Validator for the signin response schema
     */
    _signinResponseValidator = new Ajv.default(AJV_CONF).compile(SigninResponseSchema);
    /**
     * Validator for MFA Device Response schema
     */
    _resendMFADeviceResponseValidator = new Ajv.default(AJV_CONF).compile(ResendMFADeviceResponseSchema);
    /**
     * Validator for MFA Phone Response schema
     */
    _resendMFAPhoneResponseValidator = new Ajv.default(AJV_CONF).compile(ResendMFAPhoneResponseSchema);
    /**
     * Validator for the trust response schema
     */
    _trustResponseValidator = new Ajv.default(AJV_CONF).compile(TrustResponseSchema);
    /**
     * Validator for the iCloud setup response schema
     */
    _setupResponseValidator = new Ajv.default(AJV_CONF).compile(SetupResponseSchema);
    /**
     * Validator for the PCS response schema
     */
    _pcsResponseValidator = new Ajv.default(AJV_CONF).compile(PCSResponseSchema);
    /**
     * Validator for the iCloud photos setup response schema
     */
    _photosSetupResponseValidator = new Ajv.default(AJV_CONF).compile(PhotosSetupResponseSchema);
    /**
     * Generic validation function
     * @param validator - Uses the pre-configured ajv validator to validate the data
     * @param errorStruct - The error struct to throw, in case validation fails
     * @param data - The data to validate
     * @param additionalValidations - Optional additional validation functions
     * @returns The validated data
     * @throws The error struct with context and message if validation fails
     */
    validate(validator, errorStruct, data, ...additionalValidations) {
        if (validator(data) && additionalValidations.every(validation => validation(data))) {
            return data;
        }
        if (validator.errors) {
            throw new iCPSError(errorStruct)
                .addMessage(`${validator.errors[0].message} (${validator.errors[0].instancePath})`)
                .addContext(`data`, data);
        }
        throw new iCPSError(errorStruct)
            .addMessage(`Additional validations failed`)
            .addContext(`data`, data);
    }
    /**
     * Validates the provided data string against the resource file schema
     * @param data - The data to validate
     * @throws An error if the data cannot be parsed
     * @returns The parsed ResourceFile data
     */
    validateResourceFile(data) {
        return this.validate(this._resourceFileValidator, VALIDATOR_ERR.RESOURCE_FILE, data);
    }
    /**
     * Validates the response from the signin request
     * @param data - The data to validate
     * @returns A validated SigninResponse object
     * @throws An error if the data cannot be validated
     */
    validateSigninResponse(data) {
        return this.validate(this._signinResponseValidator, VALIDATOR_ERR.SIGNIN_RESPONSE, data, (data) => data.headers[`set-cookie`].filter(cookieString => cookieString.startsWith(COOKIE_KEYS.AASP)).length <= 1);
    }
    /**
     * Validates the response from resending the MFA code via a device
     * @param data - The data to validate
     * @returns A validated ResendMFADeviceResponse object
     * @throws An error if the data cannot be validated
     */
    validateResendMFADeviceResponse(data) {
        return this.validate(this._resendMFADeviceResponseValidator, VALIDATOR_ERR.RESEND_MFA_DEVICE_RESPONSE, data);
    }
    /**
     * Validates the response from resending the MFA code via a phone
     * @param data - The data to validate
     * @returns A validated ResendMFAPhoneResponse object
     * @throws An error if the data cannot be validated
     */
    validateResendMFAPhoneResponse(data) {
        return this.validate(this._resendMFAPhoneResponseValidator, VALIDATOR_ERR.RESEND_MFA_PHONE_RESPONSE, data);
    }
    /**
     * Validates the response from the trust request
     * @param data - The data to validate
     * @returns A validated TrustResponse object
     * @throws An error if the data cannot be validated
     */
    validateTrustResponse(data) {
        return this.validate(this._trustResponseValidator, VALIDATOR_ERR.TRUST_RESPONSE, data);
    }
    /**
     * Validates the response from the setup request
     * @param data - The data to validate
     * @returns A validated SetupResponse object
     * @throws An error if the data cannot be validated
     */
    validateSetupResponse(data) {
        return this.validate(this._setupResponseValidator, VALIDATOR_ERR.SETUP_RESPONSE, data, (data) => data.headers[`set-cookie`].filter(cookieString => cookieString.startsWith(COOKIE_KEYS.X_APPLE)).length > 1);
    }
    /**
     * Validates the response from the PCS acquisition request
     * @param data - The data to validate
     * @returns A validated PCSResponse object
     * @throws An error if the data cannot be validated
     */
    validatePCSResponse(data) {
        return this.validate(this._pcsResponseValidator, VALIDATOR_ERR.PCS_RESPONSE, data, (data) => data.headers[`set-cookie`].filter(cookieString => cookieString.startsWith(COOKIE_KEYS.PCS_PHOTOS) || cookieString.startsWith(COOKIE_KEYS.PCS_SHARING)).length === 2);
    }
    /**
     * Validates the response from the photos setup request
     * @param data - The data to validate
     * @returns A validated PhotosSetupResponse object
     * @throws An error if the data cannot be validated
     */
    validatePhotosSetupResponse(data) {
        return this.validate(this._photosSetupResponseValidator, VALIDATOR_ERR.PHOTOS_SETUP_RESPONSE, data);
    }
}
//# sourceMappingURL=validator.js.map
//# debugId=936c64ec-57e0-e75f-223e-5c48e1c759fe